/*
 * Modification History
 *
 * 2000-November-9	Jason Rohrer
 * Created.
 */

package ctf.agent;


import ctf.common.AgentEnvironment;
import ctf.agent.Agent;

import ctf.common.AgentAction;

import java.awt.Graphics;

/**
 * A sample agent implementing the Agent interface.
 * Your class should look similar to this, except it should be called
 * "YourNetIDAgent".
 * <BR>
 * This agent plays in a rather naive fashion.  It has no notion of
 * defense.  It heads towards the enemy flag until it gets the flag,
 * and then it heads back towards its own base.  It can avoid some 
 * obstacles, but still gets stuck permanently in certain situations.
 * It has no notion of its teammates or its enemies, so it doesn't try to 
 * avoid them at all.
 * 
 * @author Jason Rohrer
 */
public class SecretAgent extends Agent {
	
	boolean mDefence = false;
	boolean mDefenceSet = false;
	
	
	int mObstNorthCount = 0;
	int mObstSouthCount = 0;
	int mObstWestCount = 0;
	int mObstEastCount = 0;
	
	int mMaxObstHits = 5;
	boolean mCorrectNorth = false;
	double mCorrectNorthDirection = 0;
	boolean mCorrectSouth = false;
	double mCorrectSouthDirection = 0;
	boolean mCorrectEast = false;
	double mCorrectEastDirection = 0;
	boolean mCorrectWest = false;
	double mCorrectWestDirection = 0;
	
	int mMoveCount = 0;
	int mReverseCount = 0;
	boolean mReverseMode = false;
	boolean mSeenGoal = false;
	boolean mHaveFlag = false;
	
	// implements Agent.getMove() interface
	public int getMove( AgentEnvironment inEnvironment ) {
		mMoveCount++;
		if( !mDefenceSet ) {
			// examine starting position to decide if we should be defence
			
			if( inEnvironment.isBaseNorth( 
				inEnvironment.OUR_TEAM, false ) ) {
				mDefence = true;
				}
			else {
				mDefence = false;
				}
			
			mDefenceSet = true;
			}
		
		// booleans describing direction of goal
		// goal is either enemy flag, or our base
		boolean goalNorth = false;
		boolean goalSouth = false;
		boolean goalEast = false;
		boolean goalWest = false;
		boolean avoidEnemy = false;
		boolean attackEnemy = false;	// true if we should attack any enemy
		
		// check for immediate obstacles blocking our path (or our teammates)	
		boolean obstNorth = inEnvironment.isObstacleNorthImmediate() ||
			inEnvironment.isAgentNorth( inEnvironment.OUR_TEAM, true );
		boolean obstSouth = inEnvironment.isObstacleSouthImmediate() ||
			inEnvironment.isAgentSouth( inEnvironment.OUR_TEAM, true );
		boolean obstEast = inEnvironment.isObstacleEastImmediate() ||
			inEnvironment.isAgentEast( inEnvironment.OUR_TEAM, true );
		boolean obstWest = inEnvironment.isObstacleWestImmediate() ||
			inEnvironment.isAgentWest( inEnvironment.OUR_TEAM, true );
		

				
		if( !inEnvironment.hasFlag() ) {
			// this agent doesn't have the flag
			mHaveFlag = false;
			// we are also blocked by our base when we're not carrying the flag
			obstNorth = obstNorth 
				|| inEnvironment.isFlagNorth( inEnvironment.OUR_TEAM, true ) 
				&& inEnvironment.isBaseNorth( inEnvironment.OUR_TEAM, true )
				&& !inEnvironment.isAgentNorth( inEnvironment.ENEMY_TEAM, true );
			obstSouth = obstSouth 
				|| inEnvironment.isFlagSouth( inEnvironment.OUR_TEAM, true ) 
				&& inEnvironment.isBaseSouth( inEnvironment.OUR_TEAM, true )
				&& !inEnvironment.isAgentSouth( inEnvironment.ENEMY_TEAM, true );
			obstEast = obstEast 
				|| inEnvironment.isFlagEast( inEnvironment.OUR_TEAM, true ) 
				&& inEnvironment.isBaseEast( inEnvironment.OUR_TEAM, true )
				&& !inEnvironment.isAgentEast( inEnvironment.ENEMY_TEAM, true );
			obstWest = obstWest 
				|| inEnvironment.isFlagWest( inEnvironment.OUR_TEAM, true ) 
				&& inEnvironment.isBaseWest( inEnvironment.OUR_TEAM, true )
				&& !inEnvironment.isAgentWest( inEnvironment.ENEMY_TEAM, true );
			
			// enemy has flag
			if( inEnvironment.hasFlag( inEnvironment.ENEMY_TEAM ) ) {
				// try to tag the flag-carrying enemy
				
				// make goal the our flag
				goalNorth = inEnvironment.isFlagNorth( 
					inEnvironment.OUR_TEAM, false );
		
				goalSouth = inEnvironment.isFlagSouth( 
					inEnvironment.OUR_TEAM, false );
		
				goalEast = inEnvironment.isFlagEast( 
					inEnvironment.OUR_TEAM, false );
		
				goalWest = inEnvironment.isFlagWest( 
					inEnvironment.OUR_TEAM, false );
				
				}
			// our team has flag
			else if( inEnvironment.hasFlag( inEnvironment.OUR_TEAM ) ) {
				// go after the enemy to block for our teammate
				attackEnemy = true;
				}
			
			// nobody has a flag right now
			else {	
				
				if( mDefence ) {
					attackEnemy = true;
					}
				else {	// offence
					// make goal the enemy flag
					goalNorth = inEnvironment.isFlagNorth( 
						inEnvironment.ENEMY_TEAM, false );

					goalSouth = inEnvironment.isFlagSouth( 
						inEnvironment.ENEMY_TEAM, false );

					goalEast = inEnvironment.isFlagEast( 
						inEnvironment.ENEMY_TEAM, false );

					goalWest = inEnvironment.isFlagWest( 
						inEnvironment.ENEMY_TEAM, false );

					avoidEnemy = true;
					}
				}
			}	
		else {
			// we have enemy flag.
			if( !mHaveFlag ) {
				mHaveFlag = true;
				// plant mine on enemy base before leaving
				return AgentAction.PLANT_HYPERDEADLY_PROXIMITY_MINE;
				}
			// make goal our base
			goalNorth = inEnvironment.isBaseNorth( 
				inEnvironment.OUR_TEAM, false );
		
			goalSouth = inEnvironment.isBaseSouth( 
				inEnvironment.OUR_TEAM, false );
		
			goalEast = inEnvironment.isBaseEast( 
				inEnvironment.OUR_TEAM, false );
		
			goalWest = inEnvironment.isBaseWest( 
				inEnvironment.OUR_TEAM, false );
			
			avoidEnemy = true;
			}
		/*
		if( !mReverseMode && mMoveCount > 25 ) {
			mReverseMode = true;
			mMoveCount = 0;
			}
		if( mReverseMode ) {
			boolean temp = goalNorth;
			goalNorth = goalEast;
			goalEast = temp;
			
			temp = goalWest;
			goalWest = goalSouth;
			goalSouth = temp;
			
			mMoveCount++;
			if( mMoveCount > 15 ) {
				mReverseMode = false;
				mMoveCount = 0;
				}
			}
		*/
		/*if( avoidEnemy ) {
			// but avoid one enemy that is immediately near us
			if( goalWest ) {
				boolean immediate = !( !goalNorth && !goalSouth );
				if( inEnvironment.isAgentWest( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalWest = false;
					// move north or south to avoid
					if( !obstNorth ) {
						goalNorth = true;
						goalSouth = false;
						}
					else if( !obstSouth ) {
						goalNorth = false;
						goalSouth = true;
						}
					else {
						goalNorth = false;
						goalSouth = false;
						goalEast = true;
						}
					}
				}
			else if( goalEast ) {
				boolean immediate = !( !goalNorth && !goalSouth );
				if( inEnvironment.isAgentEast( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalEast = false;
					// move north or south to avoid
					if( !obstNorth ) {
						goalNorth = true;
						goalSouth = false;
						}
					else if( !obstSouth ) {
						goalNorth = false;
						goalSouth = true;
						}
					else {
						goalNorth = false;
						goalSouth = false;
						goalWest = true;
						}
					}
				}
			if( goalSouth ) {
				boolean immediate = !( !goalEast && !goalWest );
				if( inEnvironment.isAgentSouth( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalSouth = false;
					// move east or west to avoid
					if( !obstEast ) {
						goalEast = true;
						goalWest = false;
						}
					else if( !obstWest ) {
						goalEast = false;
						goalWest = true;
						}
					else {
						goalEast = false;
						goalWest = false;
						goalNorth = true;
						}
					}
				}
			else if( goalNorth ) {
				boolean immediate = !(!goalEast && !goalWest);
				if( inEnvironment.isAgentNorth( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalNorth = false;
					// move east or west to avoid
					if( !obstEast ) {
						goalEast = true;
						goalWest = false;
						}
					else if( !obstWest ) {
						goalEast = false;
						goalWest = true;
						}
					else {
						goalEast = false;
						goalWest = false;
						goalSouth = true;
						}
					}
				}		
			}	// end enemy avoidance case
		*/
		if( avoidEnemy ) {
			obstNorth = obstNorth || inEnvironment.isAgentNorth( 
				inEnvironment.ENEMY_TEAM, true );
			obstSouth = obstSouth || inEnvironment.isAgentSouth( 
				inEnvironment.ENEMY_TEAM, true );
			obstEast = obstEast || inEnvironment.isAgentEast( 
				inEnvironment.ENEMY_TEAM, true );
			obstWest = obstWest || inEnvironment.isAgentWest( 
				inEnvironment.ENEMY_TEAM, true );
			}
			
		if( attackEnemy ) {
			// make goal the enemy
			goalNorth = inEnvironment.isAgentNorth( 
				inEnvironment.ENEMY_TEAM, false );

			goalSouth = inEnvironment.isAgentSouth( 
				inEnvironment.ENEMY_TEAM, false );

			goalEast = inEnvironment.isAgentEast( 
				inEnvironment.ENEMY_TEAM, false );

			goalWest = inEnvironment.isAgentWest( 
				inEnvironment.ENEMY_TEAM, false );

			if( goalNorth && goalSouth ) {
				// pick one...

				// if immediate in either direction, pick that one
				if( inEnvironment.isAgentNorth( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalSouth = false;
					mSeenGoal = true;
					}
				else if( inEnvironment.isAgentSouth( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalNorth = false;
					mSeenGoal = true;
					}
				else {
					// pick one randomly
					if( Math.random() >= 0.5  ) {
						goalNorth = false;
						}
					else {
						goalSouth = false;
						}	
					}
				}
			if( goalEast && goalWest ) {
				// pick one...

				// if immediate in either direction, pick that one
				if( inEnvironment.isAgentEast( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalWest = false;
					mSeenGoal = true;
					}
				else if( inEnvironment.isAgentWest( 
					inEnvironment.ENEMY_TEAM, true ) ) {
					goalEast = false;
					mSeenGoal = true;
					}
				else {
					// pick one randomly
					if( Math.random() >= 0.5  ) {
						goalEast = false;
						}
					else {
						goalWest = false;
						}	
					}
				}
			}	// end enemy attack case
		
		
		
		// now we have direction booleans for our goal	
		
		// handle stuck corrections cases
		
		if( !mCorrectNorth && obstNorth ) {
			mObstNorthCount++;
			if( mObstNorthCount > mMaxObstHits ) {
				mCorrectNorth = true;
				mObstNorthCount = 0;
				mCorrectNorthDirection = Math.random();
				}
			}
		if( !mCorrectSouth && obstSouth ) {
			mObstSouthCount++;
			if( mObstSouthCount > mMaxObstHits ) {
				mCorrectSouth = true;
				mObstSouthCount = 0;
				mCorrectSouthDirection = Math.random();
				}
			}
		if(  !mCorrectEast && obstEast ) {
			mObstEastCount++;
			if( mObstEastCount > mMaxObstHits ) {
				mCorrectEast = true;
				mObstEastCount = 0;
				mCorrectEastDirection = Math.random();
				}
			}
		if(  !mCorrectWest && obstWest ) {
			mObstWestCount++;
			if( mObstWestCount > mMaxObstHits ) {
				mCorrectWest = true;
				mObstWestCount = 0;
				mCorrectWestDirection = Math.random();
				}
			}
		
		
		if( mCorrectNorth ) {
			if( goalNorth ) {
				if( mObstNorthCount < mMaxObstHits ) {
					goalNorth = false;
					if( mCorrectNorthDirection < 0.33 ) {
						goalEast = true;
						goalWest = false;
						}
					else if( mCorrectNorthDirection < 0.66 ) {
						goalWest = true;
						goalEast = false;
						}
					else {
						goalSouth = true;
						}	
					mObstNorthCount++;
					}
				else {
					mCorrectNorth = false;
					mObstNorthCount = 0;
					}
				}
			else {
				mCorrectNorth = false;
				mObstNorthCount = 0;
				}
			}
		
		else if( mCorrectSouth ) {
			if( goalSouth ) {
				if( mObstSouthCount < mMaxObstHits ) {
					goalSouth = false;
					if( mCorrectSouthDirection < 0.33 ) {
						goalEast = true;
						goalWest = false;
						}
					else if( mCorrectSouthDirection < 0.66 ) {
						goalWest = true;
						goalEast = false;
						}
					else {
						goalNorth = true;
						}
					mObstSouthCount++;
					}
				else {
					mCorrectSouth = false;
					mObstSouthCount = 0;
					}
				}
			else {
				mCorrectSouth = false;
				mObstSouthCount = 0;
				}
			}
		
		if( mCorrectEast ) {
			if( goalEast ) {
				if( mObstEastCount < mMaxObstHits ) {
					goalEast = false;
					if( mCorrectEastDirection  < 0.33 ) {
						goalNorth = true;
						goalSouth = false;
						}
					else if( mCorrectEastDirection  < 0.66 ) {
						goalSouth = true;
						goalNorth = false;
						}
					else {
						goalWest = true;
						}
					mObstEastCount++;
					}
				else {
					mCorrectEast = false;
					mObstEastCount = 0;
					}
				}
			else {
				mCorrectEast = false;
				mObstEastCount = 0;
				}
			}
		else if( mCorrectWest ) {
			if( goalWest ) {
				if( mObstWestCount < mMaxObstHits ) {
					goalWest = false;
					if( mCorrectWestDirection  < 0.33 ) {
						goalNorth = true;
						goalSouth = false;
						}
					else if( mCorrectWestDirection  < 0.66 ) {
						goalSouth = true;
						goalNorth = false;
						}
					else {
						goalEast = true;
						}
					mObstWestCount++;
					}
				else {
					mCorrectWest = false;
					mObstWestCount = 0;
					}
				}
			else {
				mCorrectWest = false;
				mObstWestCount = 0;
				}
			}
		
		// if the goal is north only, and we're not blocked
		if( goalNorth && ! goalEast && ! goalWest && !obstNorth ) {
			// move north
			return AgentAction.MOVE_NORTH;
			}
		
			
		// if goal both north and east
		if( goalNorth && goalEast ) {
			// pick north or east for move with 50/50 chance
			if( Math.random() < 0.5 && !obstNorth ) {
				return AgentAction.MOVE_NORTH;
				}
			if( !obstEast ) {	
				return AgentAction.MOVE_EAST;
				}
			if( !obstNorth ) {	
				return AgentAction.MOVE_NORTH;
				}
			}	
			
		// if goal both north and west	
		if( goalNorth && goalWest ) {
			// pick north or west for move with 50/50 chance
			if( Math.random() < 0.5 && !obstNorth ) {
				return AgentAction.MOVE_NORTH;
				}
			if( !obstWest ) {	
				return AgentAction.MOVE_WEST;
				}
			if( !obstNorth ) {	
				return AgentAction.MOVE_NORTH;
				}	
			}
		
		// if the goal is south only, and we're not blocked
		if( goalSouth && ! goalEast && ! goalWest && !obstSouth ) {
			// move south
			return AgentAction.MOVE_SOUTH;
			}
		
		// do same for southeast and southwest as for north versions	
		if( goalSouth && goalEast ) {
			if( Math.random() < 0.5 && !obstSouth ) {
				return AgentAction.MOVE_SOUTH;
				}
			if( !obstEast ) {
				return AgentAction.MOVE_EAST;
				}
			if( !obstSouth ) {
				return AgentAction.MOVE_SOUTH;
				}
			}
				
		if( goalSouth && goalWest && !obstSouth ) {
			if( Math.random() < 0.5 ) {
				return AgentAction.MOVE_SOUTH;
				}
			if( !obstWest ) {
				return AgentAction.MOVE_WEST;
				}
			if( !obstSouth ) {
				return AgentAction.MOVE_SOUTH;
				}
			}
		
		// if the goal is east only, and we're not blocked
		if( goalEast && !obstEast ) {
			return AgentAction.MOVE_EAST;
			}
		
		// if the goal is west only, and we're not blocked	
		if( goalWest && !obstWest ) {
			return AgentAction.MOVE_WEST;
			}	
		
		double moveChoice = Math.random();
		
		// otherwise, make any unblocked move
		if( moveChoice < 0.25 && !obstNorth ) {
			return AgentAction.MOVE_NORTH;
			}
		else if( moveChoice < 0.5 && !obstSouth ) {
			return AgentAction.MOVE_SOUTH;
			}
		else if( moveChoice < 0.75 && !obstEast ) {
			return AgentAction.MOVE_EAST;
			}
		else if( !obstWest ) {
			return AgentAction.MOVE_WEST;
			}	
		else {
			// completely blocked!
			return AgentAction.DO_NOTHING;
			}	
		}
	}
